#include <math.h>
#include <qpushbutton.h>
#include <qtextedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qpainter.h>
#include <qaction.h>
#include <qtable.h>
#include <qtabwidget.h>
#include <qbuttongroup.h>
#include <qmenubar.h>
#include <qinputdialog.h>
#include <qprinter.h>
#include <qsimplerichtext.h>
#include <qstatusbar.h>
#include <qpaintdevicemetrics.h> 
#include <qmessagebox.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qtextcodec.h>
#include <qstringlist.h>
#include <qsplitter.h>
#include <qscrollview.h>
//--------------------------------------------------------------------------------------
#include "gui.h"
//--------------------------------------------------------------------------------------
#include "structmodule.h"
#include "visualtab.h"
#include "guiobject.h"

#include "siegenerator.h"
#include "sieterminator.h"
#include "siedevice.h"
#include "siequeue.h"
#include "sieunpacker.h"
#include "siepacker.h"
#include "sieselector.h"

#include "sietypes.h"
#include "sieinitparser.h"

//----------------------------------------------------------
GUI::GUI() : GUIBase(), guiObjectSize(80,30),
		generatorBackgroundColor(Qt::red),generatorTextColor(Qt::black),
		queueBackgroundColor(Qt::blue),queueTextColor(Qt::cyan),
		deviceBackgroundColor(Qt::yellow),deviceTextColor(Qt::black),
		packerBackgroundColor(Qt::magenta),packerTextColor(Qt::black),
		unpackerBackgroundColor(Qt::gray),unpackerTextColor(Qt::black),
		selectorBackgroundColor(Qt::green),selectorTextColor(Qt::black),
		terminatorBackgroundColor(Qt::black),terminatorTextColor(Qt::white),
		module(0),guiObjectsList(0),
			guiObjectConnectionsList(0),activeObject(0),connectionObject1(0),
			newObjectType(-1),isSetConnection(false),isObjectMustDelete(false),
			generatorsNum(1),queuesNum(1),devicesNum(1),packersNum(1),
			unpackersNum(1),selectorsNum(1),terminatorsNum(1)

{
//	pbStart->setEnabled(false);
	fileSaveAsAction->setEnabled(false);
	backgroundColor=paletteBackgroundColor();//Qt::lightGray;
//	imitationStartAction->addTo(toolBar2);
//	fileExitAction->addTo(toolBar2);
	
	MenuBar->changeItem(1,"&");
	MenuBar->changeItem(2,"&");
	MenuBar->changeItem(3,"&");
	MenuBar->changeItem(4,"&");
	tbParams->horizontalHeader()->hide();
	QFont font("Arial",12);//tbParams->verticalHeader()->font();
	//font.setPointSize(12);
	tbParams->verticalHeader()->setFont(font);
//	tbParams->verticalHeader()->setStretchEnabled(false); 
	tbParams->verticalHeader()->setResizeEnabled(false);
	tbParams->setTopMargin(1);
	tbParams->setLeftMargin(50);
//	tbParams->setNumRows(1);
	tbParams->verticalHeader()->setLabel(0,"");
	connect(tbParams,SIGNAL(valueChanged(int,int)),this,SLOT(slotActiveObjectParamsChanged(int)));
	
	bgNewObjects->setColumnLayout(10,Qt::Vertical);
	bgNewObjects->setExclusive(true);
	QPushButton* pb=new QPushButton("",bgNewObjects,"pbNewGenerator");
	pb->setToggleButton(TRUE);
	pb->setPaletteBackgroundColor(generatorBackgroundColor);
	pb=new QPushButton("",bgNewObjects,"pbNewQueue");
	pb->setToggleButton(TRUE);
	pb->setPaletteBackgroundColor(queueBackgroundColor);
	pb->setPaletteForegroundColor(queueTextColor);
	pb=new QPushButton("",bgNewObjects,"pbNewDevice");
	pb->setToggleButton(TRUE);
	pb->setPaletteBackgroundColor(deviceBackgroundColor);
	pb=new QPushButton("",bgNewObjects,"pbNewPacker");
	pb->setToggleButton(TRUE);
	pb->setPaletteBackgroundColor(packerBackgroundColor);
	pb=new QPushButton("",bgNewObjects,"pbNewUnpacker");
	pb->setToggleButton(TRUE);
	pb->setPaletteBackgroundColor(unpackerBackgroundColor);
	pb=new QPushButton("",bgNewObjects,"pbNewSelector");
	pb->setToggleButton(TRUE);
	pb->setPaletteBackgroundColor(selectorBackgroundColor);
	pb=new QPushButton("",bgNewObjects,"pbNewTerminator");
	pb->setToggleButton(TRUE);
	pb->setPaletteBackgroundColor(terminatorBackgroundColor);
	pb->setPaletteForegroundColor(terminatorTextColor);
	bgNewObjects->addSpace(1);
	pb=new QPushButton("",bgNewObjects,"pbConnection");
	pb->setToggleButton(TRUE);
	pb=new QPushButton("",bgNewObjects,"pbDelete");
	pb->setToggleButton(TRUE);
	
	
	visualTab=new VisualTab(tabModel,"visualTab",&pixBuffer);
	codeTab=new QTextEdit(tabModel,"codeTab");
	statisticTab=new QTextEdit(tabModel,"statisticTab");
	
	codeTab->setReadOnly(true);
	statisticTab->setReadOnly(true);
	
	visualTabScroll=new QScrollView(tabModel);
	tabModel->removePage(tabModel->currentPage());
	tabModel->addTab(codeTab,"");
	tabModel->addTab(visualTabScroll,"");
	tabModel->addTab(statisticTab,"");
	
	visualTabScroll->addChild(visualTab);
	tabModel->setCurrentPage(1);
	
//	teOutput->setMaxLogLines(600);
	
	updateBuffer();
}

void GUI::initWidget()
{
//	QSize sz=teOutput->size();
//	sz.setHeight(100);
//	teOutput->resize(sz);

}
//----------------------------------------------------------

GUI::~GUI()
{
	qDebug("~GUI");
	slotClose();
}

//----------------------------------------------------------
void GUI::slotTabChanged(QWidget* tab)
{
//	qDebug("changed! %ld %ld %d",tab,visualTab,bgNewObjects->isExclusive());
	
	if(tab!=visualTab)
	{
		slotActiveObjectChanged(0);
//		bgNewObjects->hide();
	}
	else
	{
		updateBuffer();
//		bgNewObjects->show();
	}
}

GuiObject* GUI::addGuiObject(int type,char* name,int x,int y)
{
	GuiProperty prop;
	prop.type=type;
	prop.width=guiObjectSize.width();
	prop.height=guiObjectSize.height();
	prop.x=x;
	prop.y=y;
	switch(type)
	{
	case SIE_GENERATOR:
		prop.backgroundColor=generatorBackgroundColor;
		prop.textColor=generatorTextColor;
		break;
	case SIE_TERMINATOR:
		prop.backgroundColor=terminatorBackgroundColor;
		prop.textColor=terminatorTextColor;
		break;
	case SIE_DEVICE:
		prop.backgroundColor=Qt::yellow;
		prop.textColor=Qt::black;
		break;
	case SIE_QUEUE:
		prop.backgroundColor=Qt::blue;
		prop.textColor=Qt::cyan;
		break;
	case SIE_PACKER:
		prop.backgroundColor=Qt::magenta;
		prop.textColor=Qt::black;
		break;
	case SIE_UNPACKER:
		prop.backgroundColor=Qt::gray;
		prop.textColor=Qt::black;
		break;
	case SIE_SELECTOR:
		prop.backgroundColor=selectorBackgroundColor;
		prop.textColor=selectorTextColor;
		break;
	}
	prop.fontSize=14;
	
	GuiObjectsList* p=guiObjectsList;
	if(p)
	{
//		qDebug("+1 GuiObject");
		while(p->next) p=p->next;
		p->next=new GuiObjectsList;
		p=p->next;
	}
	else
	{
//		qDebug("new GuiObject");
		p=guiObjectsList=new GuiObjectsList;
		
	}
	
	p->next=0;
	p->obj=new GuiObject(visualTab,&prop,name);
//	connect(p->obj,SIGNAL(signalFix(GuiObject*)),this,SLOT(slotGuiObjectChanged(GuiObject*)));
	connect(p->obj,SIGNAL(signalFix(GuiObject*)),SLOT(updateBuffer()));
	connect(p->obj,SIGNAL(signalPress(GuiObject*)),SLOT(slotActiveObjectChanged(GuiObject*)));
	connect(this,SIGNAL(signalSetInactive()),p->obj,SLOT(slotSetInactive()));
	p->obj->show();
	return p->obj;
}

//----------------------------------------------------------

void GUI::slotPrepareToAddObject(int id)
{
	
	qDebug("id=%d",id);
	QPushButton* pb;
	if(id<0)
	{
		pb=dynamic_cast<QPushButton*>(bgNewObjects->find(0));
		if(!pb->isOn()) 
			pb->toggle();
		pb->toggle();
		return;
	}
		
	pb=dynamic_cast<QPushButton*>(bgNewObjects->find(id));
	if(id==7) 
	{
		if(!pb->isOn()) 
			isSetConnection=false;
		else
		{
			newObjectType=-1;
			if(!activeObject) 
			{
				isSetConnection=false;
				pb->toggle();
				return;
			}
			isSetConnection=true;
		}
		return;
	}
	else if(id==8)
	{
		newObjectType=-1;
		isSetConnection=false;
		if(!activeObject)
		{
			pb->toggle();
			return;
		}
		GuiObjectsList* d1,*d=guiObjectsList;
		if(d->obj==activeObject)
			d=0;
		else
		{
			while(d)
			{
				if(d->next->obj==activeObject) break;
				d=d->next;
			}
		}
		if(d)
		{
			d1=d->next;
			d->next=d->next->next;
			d=d1;
		}
		else
		{
			d=guiObjectsList;
			guiObjectsList=d->next;
		}
		
		GuiObjectConnectionsList* pp=guiObjectConnectionsList;
		GuiObjectConnectionsList* dd,*prevpp=0;
		while(pp)
		{
			if((pp->obj1==d->obj)||(pp->obj2==d->obj))
			{
				if(prevpp)
					prevpp->next=pp->next;
				else
					guiObjectConnectionsList=pp->next;
				dd=pp;
				pp=pp->next;
				delete dd;
			}
			else
			{
				prevpp=pp;
				pp=pp->next;
			}
		}
		
			
		
		delete d->obj;
		delete d;
	
		isObjectMustDelete=false;
		slotActiveObjectChanged(0);
		pb->toggle();
		updateBuffer();
		return;
	}
	
	if(pb->isOn()) 
		newObjectType=id;
	else
		newObjectType=-1;
}

//----------------------------------------------------------
void GUI::slotFind()
{
	bool ok;
	QString text=QInputDialog::getText(name()," :",QLineEdit::Normal,QString::null,&ok,this);
	if(ok&&!text.isEmpty()) 
	{
		slotActiveObjectChanged(findGuiObject(text));
	} 
}

//----------------------------------------------------------
void GUI::slotActiveObjectChanged(GuiObject* obj)
{
	emit signalSetInactive();
	
	if(isSetConnection) 
	{
		
		isSetConnection=false;
		
		addGuiObjectConnection(activeObject->name(),obj->name());
		module->setConnection(activeObject->name(),obj->name());
		
		//connectionObject1=0;
		slotPrepareToAddObject(-1);
		updateBuffer();
	}
	
	
	activeObject=obj;
	
	if(!activeObject)
	{
		tbParams->setNumRows(0);
		return;
	}
	
	

	
	
	
	
	
	
	
	
	
	activeObject->slotSetActive();
	tbParams->setNumRows(1);
	tbParams->verticalHeader()->setLabel(0," ");
	tbParams->setText(0,0,activeObject->name());
//	tbParams->verticalHeader()->setLabel(0,"");
	sieObject* object=module->find(obj->name());
	if(!object) return;
	
	sieGenerator* gobj;
	sieQueue* qobj;
	sieDevice* dobj;
	siePacker* pobj;
	sieUnpacker* uobj;
	sieSelector* sobj;
	sieTerminator* tobj;
	
	QStringList* strlist;
	QComboTableItem* item;
	int i,k;
	switch(object->objectType())
	{
	case SIE_GENERATOR:
		tbParams->setNumRows(9);
		gobj=dynamic_cast<sieGenerator*>(object);	
		tbParams->verticalHeader()->setLabel(1,"");
		tbParams->setText(1,0,QString("%1").arg(gobj->generationTime()));
		tbParams->verticalHeader()->setLabel(2,"");
		tbParams->setText(2,0,QString("%1").arg(gobj->dispersion()));
		tbParams->verticalHeader()->setLabel(3,"");
		tbParams->setText(3,0,QString("%1").arg(gobj->generationNumber()));
		
		tbParams->verticalHeader()->setLabel(4," ");
		tbParams->setText(4,0,QString("%1").arg(gobj->transactType()));
		tbParams->verticalHeader()->setLabel(5," -");
		tbParams->setText(5,0,QString("%1").arg(gobj->cost()));
		tbParams->verticalHeader()->setLabel(6," -");
		tbParams->setText(6,0,QString("%1").arg(gobj->costDisp()));
		tbParams->verticalHeader()->setLabel(7,".  -");
		tbParams->setText(7,0,QString("%1").arg(gobj->lifeLimit()));
		tbParams->verticalHeader()->setLabel(8,". . ");
		tbParams->setText(8,0,QString("%1").arg(gobj->lifeLimitDisp()));

		break;
	case SIE_QUEUE:	
		tbParams->setNumRows(2);
		qobj=dynamic_cast<sieQueue*>(object);	
		tbParams->verticalHeader()->setLabel(1,"");
		tbParams->setText(1,0,QString("%1").arg(qobj->maxNumber()));
		break;
	
	case SIE_DEVICE:
		tbParams->setNumRows(3);
		dobj=dynamic_cast<sieDevice*>(object);	
		tbParams->verticalHeader()->setLabel(1,"");
		tbParams->setText(1,0,QString("%1").arg(dobj->delay()));
		tbParams->verticalHeader()->setLabel(2,"");
		tbParams->setText(2,0,QString("%1").arg(dobj->dispersion()));
		break;
	case SIE_PACKER:	
		tbParams->setNumRows(2);
		pobj=dynamic_cast<siePacker*>(object);	
		tbParams->verticalHeader()->setLabel(1,"");
		tbParams->setText(1,0,QString("%1").arg(pobj->transactType()));
		break;
	case SIE_UNPACKER:	
		tbParams->setNumRows(2);
		uobj=dynamic_cast<sieUnpacker*>(object);	
		tbParams->verticalHeader()->setLabel(1,"");
		tbParams->setText(1,0,QString("%1").arg(uobj->transactType()));
		break;
	case SIE_SELECTOR:
		sobj=dynamic_cast<sieSelector*>(object);	
		k=sobj->destinationNum();
		tbParams->setNumRows(2);
		tbParams->verticalHeader()->setLabel(1,"");
		strlist=new QStringList;
		strlist->append("0");
		strlist->append("1");
		item=new QComboTableItem(tbParams,*strlist);
		tbParams->setItem(1,0,item);
		item->setCurrentItem(sobj->selectorType());
		//tbParams->setText(1,0,QString("%1").arg(sobj->selectorType()));
		if(sobj->selectorType()==0)
		{
			tbParams->setNumRows(2+k);
			for(i=0;i<k;i++)
			{
				tbParams->verticalHeader()->setLabel(2+i,sobj->destinationName(i));
				tbParams->setText(2+i,0,QString("%1").arg(sobj->destinationProbability(i)));
			}
		}
		else if(sobj->selectorType()==1)
		{
		}
		break;
	case SIE_TERMINATOR: 
		tbParams->setNumRows(2);
		tobj=dynamic_cast<sieTerminator*>(object);	
		tbParams->verticalHeader()->setLabel(1,"");
		tbParams->setText(1,0,QString("%1").arg(tobj->decrementNumber()));
		break;
	}
	
}

//----------------------------------------------------------

void GUI::slotActiveObjectParamsChanged(int k)
{
	if(!activeObject) return;
	qDebug("k=%d  %s",k,tbParams->text(k,0).ascii());
	
	sieObject* object=module->find(activeObject->name());
	if(!object) return;
	if(k==0)
	{
		activeObject->setInitName(tbParams->text(k,0));
		object->setName(tbParams->text(k,0));
		activeObject->updateBuffer();
		return;
	}
	
	
	sieGenerator* gobj;
	sieQueue* qobj;
	sieDevice* dobj;
	siePacker* pobj;
	sieUnpacker* uobj;
	sieTerminator* tobj;
	sieSelector* sobj;
	
	sieTransact* trans;
	switch(object->objectType())
	{
		case SIE_GENERATOR:
			qDebug("generator");
			gobj=dynamic_cast<sieGenerator*>(object);	
			if(k==1)
				gobj->setGenerationTime(tbParams->text(1,0).toInt());
			else if(k==2)
				gobj->setDispersion(tbParams->text(2,0).toInt());
			else if(k==3)
				gobj->setGenerationNumber(tbParams->text(3,0).toInt());
			else if(k==4)
				gobj->setGenerationType(tbParams->text(4,0).toInt());
			else 
			{
				trans=gobj->transactProperty();
				if(k==5)
				{
					trans->cost=tbParams->text(5,0).toDouble();
					gobj->setTransactProperty(trans,-1,-1.);
				}
				else if(k==6)
				{
					gobj->setTransactProperty(0,-1,tbParams->text(6,0).toDouble());
				}
				else if(k==7)
				{
					trans->lifeLimit=tbParams->text(7,0).toInt();
					gobj->setTransactProperty(trans,-1,-1.);
				}
				else if(k==8)
				{
//					qDebug("1  %d",tbParams->text(8,0).toInt());
					gobj->setTransactProperty(0,tbParams->text(8,0).toInt(),-1.);
//			qDebug("2");
				}
			}
				
			break;
		case SIE_QUEUE:	
			qobj=dynamic_cast<sieQueue*>(object);	
			if(k==1)
				qobj->setMaxNumber(tbParams->text(1,0).toInt());
			break;
		case SIE_DEVICE:
			dobj=dynamic_cast<sieDevice*>(object);	
			if(k==1)
				dobj->setDelay(tbParams->text(1,0).toInt());
			else if(k==2)
				dobj->setDispersion(tbParams->text(2,0).toInt());
			break;
		case SIE_PACKER:	
			pobj=dynamic_cast<siePacker*>(object);	
			if(k==1)
				pobj->setTransactType(tbParams->text(1,0).toInt());
			break;
		case SIE_UNPACKER:	
			uobj=dynamic_cast<sieUnpacker*>(object);	
			if(k==1)
				uobj->setTransactType(tbParams->text(1,0).toInt());
			break;
		case SIE_SELECTOR:	
			sobj=dynamic_cast<sieSelector*>(object);	
			if(k==1)
			{
				sobj->setSelectorType(tbParams->text(1,0).toInt());
				qDebug("text=%d",tbParams->text(1,0).toInt());
				slotActiveObjectChanged(activeObject);
			}
			else 
			{
				sobj->setDestinationProbability(k-2,tbParams->text(k,0).toInt());
			}
			
			break;
		case SIE_TERMINATOR: 
			tobj=dynamic_cast<sieTerminator*>(object);	
			if(k==1)
				tobj->setDecrementNumber(tbParams->text(1,0).toInt());
			break;
	}
}

//----------------------------------------------------------

void GUI::slotGuiObjectChanged(GuiObject* obj)
{
	obj=obj;
//	updateBuffer();
}

//----------------------------------------------------------

void GUI::mousePressEvent(QMouseEvent* pe)
{
	if(isSetConnection) return;
	slotActiveObjectChanged(0);

//	tbParams->verticalHeader()->setLabel(0,"");
//	tbParams->setText(0,0,"");
	
	if(newObjectType<0) return;
	
	if(!module) slotNew();
	
	char name[NAMESIZE];
	int type=0;
	int tgen=1;
	int ngen=1;
	int maxnum=10;
	int delay=2;
	int disp=1;
	int numdec=0;
	
	sieTransact tr;
	tr.type=type;
	tr.lifeLimit=9999;
	tr.cost=250.;
	int trLLD=0;
	double trCD=50.;
	
	sieGenerator* gen;
	
	switch(newObjectType)
	{
	case SIE_GENERATOR:	
		sprintf(name,"%d",generatorsNum++);
		gen=dynamic_cast<sieGenerator*>(module->addObject(name,SIE_GENERATOR,tgen,disp,ngen));
		gen->setTransactProperty(&tr,trLLD,trCD);
		break;
	case SIE_QUEUE:	
		sprintf(name,"%d",queuesNum++);
		module->addObject(name,SIE_QUEUE,maxnum);
		break;
	
	case SIE_DEVICE:
		sprintf(name,"%d",devicesNum++);
		module->addObject(name,SIE_DEVICE,delay,disp);
		break;
	case SIE_PACKER:	
		sprintf(name,"%d",packersNum++);
		module->addObject(name,SIE_PACKER,type);
		break;
	case SIE_UNPACKER:	
		sprintf(name,"%d",unpackersNum++);
		module->addObject(name,SIE_UNPACKER,type);
		break;
	case SIE_SELECTOR:	
		sprintf(name,"%d",selectorsNum++);
		module->addObject(name,SIE_SELECTOR,0);
		break;
	case SIE_TERMINATOR: 
		sprintf(name,"%d",terminatorsNum++);
		module->addObject(name,SIE_TERMINATOR,numdec);
		break;
	}
	addGuiObject(newObjectType,name,
#ifdef WIN32
		pe->x()-tabModel->x()-guiObjectSize.width()/2-30,
		     pe->y()-tabModel->y()-toolBar->height()-MenuBar->height()-guiObjectSize.height()/2-20);
#endif
	qDebug("1=%d\n2=%d\n3=%d\n4=%d",pe->x(),tabModel->x(),
		visualTabScroll->contentsX(),guiObjectSize.width()/2);
	
//	QPushButton* pb=dynamic_cast<QPushButton*>(bgNewObjects->find(newObjectType));
//	pb->toggle();
	slotPrepareToAddObject(-1);
	newObjectType=-1;

}

//----------------------------------------------------------

void GUI::addGuiObjectConnection(const char* obj1,const char* obj2)
{
//	qDebug("addGuiObjectConnection");
	if(!obj1) return;
	if(!obj2) return;
	GuiObject* object1=findGuiObject(obj1);
	GuiObject* object2=findGuiObject(obj2);
	if(!object1) return;
	if(!object2) return;
	if(object1==object2) return;
//	qDebug("ee");
	GuiObjectConnectionsList* p=guiObjectConnectionsList;
	if(p)
	{
		while(p->next) p=p->next;
		p->next=new GuiObjectConnectionsList;
		p=p->next;
	}
	else
	{
		p=guiObjectConnectionsList=new GuiObjectConnectionsList;
	}
	
	p->next=0;
	p->obj1=object1;
	p->obj2=object2;
}

//----------------------------------------------------------

GuiObject* GUI::findGuiObject(const char* name)
{
//	qDebug("findGuiObject");
	GuiObjectsList* p=guiObjectsList;
	while(p) 
	{
//		printf("%s %s\n",p->obj->name(),name);
		if(strncmp(p->obj->name(),name,NAMESIZE)==0) return p->obj;
//		qDebug("%s!=%s",p->obj->name(),name);
		p=p->next;
	}
	qDebug("No");
	return 0;
}

//----------------------------------------------------------

void GUI::slotStart()
{
	teOutput->clear();
	if(!module) return;
	tabModel->setCurrentPage(1);
	module->init(cbVisual->isChecked());
	if(sbModelLife->value()>0)
		module->start(1000-slSpeed->value(),sbModelLife->value());
	else
		module->start(1000-slSpeed->value());
	statisticTab->clear();
	teOutput->append(" ");
	imitationStartAction->setVisible(false);
	imitationStopAction->setVisible(true);
}
//----------------------------------------------------------

void GUI::slotStop()
{
	module->stop();
	imitationStartAction->setVisible(true);
	imitationStopAction->setVisible(false);
	teOutput->append(" ");
}

//----------------------------------------------------------

void GUI::slotComplete()
{
	statisticTab->clear();
	statisticTab->append(module->getStatistic());
	teOutput->append(" !");
	tabModel->setCurrentPage(2);
	imitationStartAction->setVisible(true);
	imitationStopAction->setVisible(false);
}

//----------------------------------------------------------

void GUI::slotClose()
{
	GuiObjectsList* d,*p=guiObjectsList;
	while(p)
	{
		d=p;
		p=p->next;
		delete d->obj;
		delete d;
	}

	GuiObjectConnectionsList* dd,*pp=guiObjectConnectionsList;
	while(pp)
	{
		dd=pp;
		pp=pp->next;
		delete dd;
	}
	guiObjectsList=0;
	guiObjectConnectionsList=0;
	
	if(module) delete module;
	module=0;
	slotActiveObjectChanged(0);
	updateBuffer();
	codeTab->clear();
	statisticTab->clear();
	fileCloseAction->setEnabled(false);
	fileSaveAsAction->setEnabled(false);
	imitationStartAction->setEnabled(false);
	setCaption(PROGRAM_NAME);
}

//----------------------------------------------------------

void GUI::slotNew()
{
//	qDebug("slotNew");
	teOutput->clear();
	teOutput->append(QString(" "));
	init(0);
	imitationStartAction->setEnabled(true);
	fileSaveAsAction->setEnabled(true);
	fileCloseAction->setEnabled(true);
	imitationStartAction->setEnabled(true);
	updateBuffer();
	setCaption(QString("%1 - %2").arg(PROGRAM_NAME).arg("unnamed.xml"));
}

//----------------------------------------------------------

void GUI::slotOpen()
{
	QString s = QFileDialog::getOpenFileName(
			"",
			"SIE init files (*.xml);;GPSS Model (*.txt)",
			this,
			"...",
			"   " );
  
	if(s.isNull()) return;
	qDebug("s=\"%s\"",s.ascii());
	
	if(s.find("xml")==static_cast<signed int>(s.length()-3))
	{
//		teCons->clear();
		teOutput->append(QString("   \"%1\"...").arg(s));
		if(init(s)) 
		{
			sbModelLife->setValue(module->life());
			imitationStartAction->setEnabled(true);
			fileSaveAsAction->setEnabled(true);
			fileCloseAction->setEnabled(true);
			teOutput->append(" ");
			tabModel->setCurrentPage(1);
			setCaption(QString("%1 - %2").arg(PROGRAM_NAME).arg(s.section('/',-1)));
			return;
		}
		else
			teOutput->append(QString("   \"%1\"...").arg(s));
	}
	else if(s.find("txt")==static_cast<signed int>(s.length()-3))
	{
		//teCons->clear();
		teOutput->append(QString("  \"%1\"...").arg(s));
		if(convertGPSSfile(s)) 
		{
			teOutput->append(QString("   \"%1\"...").arg(s));
			if(init("tmp.xml"))
			{
				sbModelLife->setValue(module->life());
				imitationStartAction->setEnabled(true);
				fileSaveAsAction->setEnabled(true);
				fileCloseAction->setEnabled(true);
				teOutput->append(" ");
				tabModel->setCurrentPage(1);
				setCaption(QString("%1 - %2").arg(PROGRAM_NAME).arg(s.section('/',-1)));
				return;
			}
			else
				teOutput->append(QString("   \"%1\"...").arg(s));
		}
		else
			teOutput->append(QString("   \"%1\"...").arg(s));
	}
	imitationStartAction->setEnabled(false);
	fileSaveAsAction->setEnabled(false);
	fileCloseAction->setEnabled(false);
	updateBuffer();
}

//----------------------------------------------------------

void GUI::slotSaveAs()
{
	QString s = QFileDialog::getSaveFileName(
			"",
			"Xml file (*.xml)",
			this,
			" ...",
			"   " );
	
	if(s.isNull()) return;
	if(s.find(".xml")<0)
		s.append(".xml");
	
	QFile file(s);
	if(!file.open(IO_WriteOnly)) return;
		
	QTextStream stream(&file);
	//stream.setCodec(QTextCodec::codecForName("KOI8-R"));

	
	qDebug("save..");
	
	stream<<"<?xml version=\"1.0\" encoding=\"KOI8-R\"?>\n";
	stream<<"<INIT>\n";
	stream<<"\t<OBJECTS>\n";

	sieObject* p;
	ObjectsList* d;
	sieGenerator* pg;
	sieDevice* pd;
	sieTerminator* pt;
	sieQueue* pq;
	siePacker* pp;
	sieUnpacker* pu;
	sieSelector* ps;
	
	GuiObject* guiobj;
	
	std::vector<sieObject*> items=module->objects();
	std::vector<sieObject*>::iterator current;

	
	QTextCodec *codec = QTextCodec::codecForName("KOI8-R");
	QString strname;

	
	int x,y;
	for(current=items.begin();current!=items.end();++current)
	{
		p=*current;
		guiobj=findGuiObject(p->name());
		if(guiobj)
		{
			x=guiobj->x();
			y=guiobj->y();
		}
		else
			x=y=0;
		switch(p->objectType())
		{
			case SIE_GENERATOR:
				pg=dynamic_cast<sieGenerator*>(p);
				

				strname = codec->fromUnicode(pg->name());
				stream<<"\t\t<GENERATOR name=\""<<strname<<"\"";
				stream<<" tgen=\""<<pg->generationTime()<<"\"";
				stream<<" disp=\""<<pg->dispersion()<<"\"";
				stream<<" ngen=\""<<pg->generationNumber()<<"\">\n";
		
				stream<<"\t\t\t<TRANSACT type=\""<<pg->transactType()<<"\"";
				stream<<" lifelimit=\""<<pg->lifeLimit()<<"\"";
				stream<<" lifelimitdisp=\""<<pg->lifeLimitDisp()<<"\"";
				stream<<" cost=\""<<pg->cost()<<"\"";
				stream<<" costdisp=\""<<pg->costDisp()<<"\" />\n";
				
				stream<<"\t\t\t<GUI X=\""<<x<<"\" Y=\""<<y<<"\"/>\n";
				d=pg->destination();
				while(d)
				{
					strname = codec->fromUnicode(d->obj->name());
					stream<<"\t\t\t<CONNECTION destination=\""<<strname<<"\"/>\n";
					d=d->next;
				}
				stream<<"\t\t</GENERATOR>\n";
			
				break;
			case SIE_TERMINATOR:
				pt=dynamic_cast<sieTerminator*>(p);
				strname = codec->fromUnicode(pt->name());
				stream<<"\t\t<TERMINATOR name=\""<<strname<<"\"";
				if(pt->decrementNumber()>0)
				{
					stream<<" decrement=\""<<pt->decrementNumber()<<"\"";
				}
				stream<<">\n";
				stream<<"\t\t\t<GUI X=\""<<x<<"\" Y=\""<<y<<"\"/>\n";
				stream<<"\t\t</TERMINATOR>\n";
				break;
			case SIE_DEVICE:
				pd=dynamic_cast<sieDevice*>(p);
				strname = codec->fromUnicode(pd->name());
				stream<<"\t\t<DEVICE name=\""<<strname<<"\"";
				stream<<" delay=\""<<pd->delay()<<"\"";
				stream<<" disp=\""<<pd->dispersion()<<"\"";
			//stream<<" type=\""<<pd->type()<<"\"";
				stream<<">\n";
		
				stream<<"\t\t\t<GUI X=\""<<x<<"\" Y=\""<<y<<"\"/>\n";
				d=pd->destination();
				while(d)
				{
					strname = codec->fromUnicode(d->obj->name());
					stream<<"\t\t\t<CONNECTION destination=\""<<strname<<"\"/>\n";
					d=d->next;
				}
				stream<<"\t\t</DEVICE>\n";
			
				break;
			case SIE_QUEUE:
				pq=dynamic_cast<sieQueue*>(p);
				strname = codec->fromUnicode(pq->name());
				stream<<"\t\t<QUEUE name=\""<<strname<<"\"";
				stream<<" maxnum=\""<<pq->maxNumber()<<"\">\n";
		
				stream<<"\t\t\t<GUI X=\""<<x<<"\" Y=\""<<y<<"\"/>\n";
				d=pq->destination();
				while(d)
				{
					strname = codec->fromUnicode(d->obj->name());
					stream<<"\t\t\t<CONNECTION destination=\""<<strname<<"\"";
					stream<<" type=\""<<d->type<<"\"/>\n";
					d=d->next;
				}
				stream<<"\t\t</QUEUE>\n";
				break;
			case SIE_PACKER:
				pp=dynamic_cast<siePacker*>(p);
				strname = codec->fromUnicode(pp->name());
				stream<<"\t\t<PACKER name=\""<<strname<<"\"";
				stream<<" type=\""<<pp->transactType()<<"\">\n";
		
				stream<<"\t\t\t<GUI X=\""<<x<<"\" Y=\""<<y<<"\"/>\n";
				d=pp->destination();
				while(d)
				{
					strname = codec->fromUnicode(d->obj->name());
					stream<<"\t\t\t<CONNECTION destination=\""<<strname<<"\"/>\n";
					d=d->next;
				}
				stream<<"\t\t</PACKER>\n";
				break;
			case SIE_UNPACKER:
				pu=dynamic_cast<sieUnpacker*>(p);
				strname = codec->fromUnicode(pu->name());
				stream<<"\t\t<UNPACKER name=\""<<strname<<"\"";
				stream<<" type=\""<<pu->transactType()<<"\">\n";
		
				stream<<"\t\t\t<GUI X=\""<<x<<"\" Y=\""<<y<<"\"/>\n";
				d=pu->destination();
				while(d)
				{
					strname = codec->fromUnicode(d->obj->name());
					stream<<"\t\t\t<CONNECTION destination=\""<<strname<<"\"";
					stream<<" type=\""<<d->type<<"\"/>\n";
					d=d->next;
				}
				stream<<"\t\t</UNPACKER>\n";
				break;
			case SIE_SELECTOR:
				ps=dynamic_cast<sieSelector*>(p);
				strname = codec->fromUnicode(ps->name());
				stream<<"\t\t<SELECTOR name=\""<<strname<<"\"";
				stream<<" type=\""<<ps->selectorType()<<"\">\n";
		
				stream<<"\t\t\t<GUI X=\""<<x<<"\" Y=\""<<y<<"\"/>\n";
				d=ps->destination();
				while(d)
				{
					strname = codec->fromUnicode(d->obj->name());
					stream<<"\t\t\t<CONNECTION destination=\""<<strname<<"\"";
					stream<<" probability=\""<<d->probability<<"\"/>\n";
					d=d->next;
				}
				stream<<"\t\t</SELECTOR>\n";
				break;
			default:
				qDebug("UNKNOWN OBJECT!!!");
		}
	}
	
	stream<<"\t</OBJECTS>\n";
	stream<<"\t<MODEL life=\""<<sbModelLife->value()<<"\" />\n";
	stream<<"</INIT>\n";
	setCaption(QString("%1 - %2").arg(PROGRAM_NAME).arg(s.section('/',-1)));

	file.close();

}

//----------------------------------------------------------

void GUI::slotExit()
{
	delete this;
}

//----------------------------------------------------------

void GUI::loadCode(const char* filename)
{
//	qDebug("loadCode \"%s\"",filename);
	codeTab->clear();
	if(!filename) return;
//	qDebug("loadCode2");
	QString str;
	QFile file(filename);
	
	if(!file.open(IO_ReadOnly)) return;
//	qDebug("loadCode3");

	QTextStream stream(&file);
	while(!stream.atEnd())
	{
		str=stream.readLine();
		codeTab->append(str);
	}
	file.close();
}
	
//----------------------------------------------------------

bool GUI::convertGPSSfile(const char* filename)
{
	size_t i;
	int modellife=-1;
	int gk=1,qk=1,dk=1,tk=1;
	QString str,str1,tag1,tag2;
	QFile file(filename);
	QFile fileout("tmp.xml");
	
	if(file.open(IO_ReadOnly))
	{
		if(!fileout.open(IO_WriteOnly)) 
		{
			file.close();
			return false;
		}
		
		QTextStream stream(&file);
		QTextStream xml(&fileout);
		xml<<"<?xml version=\"1.0\" encoding=\"KOI8-R\"?>\n<INIT>\n\t<OBJECTS>\n";

		while(!stream.atEnd())
		{
			str=stream.readLine();
//			qDebug("str=\"%s\"",str.ascii());
			str.replace("\t"," ");
//			qDebug("str=\"%s\"",str.ascii());
			do
			{
				i=str.length();
				str.replace("  "," ");
//				qDebug("str=\"%s\"",str.ascii());
			}
			while(i!=str.length());
			if(str[0]==QChar(' ')) str.remove(0,1);	
			if(str[str.length()-1]==QChar(' ')) str.remove(str.length()-1,1);	
			//qDebug("str=\"%s\"",str.ascii());
			
			i=0;
			
			while(str[i]!=QChar(' ')) 
			{
				if(i>=str.length()) break;
				i++;
			}
			if(str.find("GENERATE")>=0)
			{
				str.remove(0,i+1);
				
				str1=str.section(',',0,0);
				i=str1.toInt();
				qDebug("str1=\"%s\" par=%d",str1.ascii(),i);
				
				
				xml<<"\t\t<GENERATOR name=\"gen"<<gk++<<"\" tgen=\""<<i<<"\"";
				str1=str.section(',',1,1);
				i=str1.toInt();
				qDebug("str1=\"%s\" par=%d",str1.ascii(),i);
				if(!str1.isEmpty())
					xml<<" disp=\""<<i<<"\"";
				xml<<">\n";
				tag1=QString("\t\t\t<CONNECTION destination=\"");
				tag2=QString("\" />\n\t\t</GENERATOR>\n");
			}
			else if(str.find("SEIZE")>=0)
			{
				str.remove(0,i+1);
				
			//	qDebug("str=\"%s\"",str.ascii());
				
				xml<<tag1<<"queue"<<qk<<tag2;
				
				xml<<"\t\t<QUEUE name=\"queue"<<qk++<<"\" maxnum=\"256\">\n";
				xml<<"\t\t\t<CONNECTION destination=\"device1\" />\n";
				xml<<"\t\t</QUEUE>\n";
				
				xml<<"\t\t<DEVICE name=\"device"<<dk++<<"\" delay=\"";
				
				tag2=QString("\" />\n\t\t</DEVICE>\n");
			}
			else if(str.find("ADVANCE")>=0)
			{
					str.remove(0,i+1);
					i=str.toInt();
					qDebug("str=\"%s\"",str.ascii());
				
					xml<<i<<"\" >\n";
					tag1=QString("\t\t\t<CONNECTION destination=\"");
			}
			else if(str.find("TERMINATE")>=0)
			{
				str.remove(0,i+1);
				i=str.toInt();
				qDebug("str=\"%s\" i=%d",str.ascii(),i);
				
				xml<<tag1<<"term"<<tk<<tag2;
				xml<<"\t\t<TERMINATOR name=\"term"<<tk++<<"\" decrement=\""<<i<<"\" />\n";
				tag1=tag2=QString("");
			}
			else if(str.find("START")>=0)
			{
				str.remove(0,i+1);
				modellife=str.toInt();
			}
			//else
			//	qDebug("don't know \"%s\"",str.ascii());
		}
		//xml<<"-----------------\n"<<tag1<<tag2;
		xml<<"\t</OBJECTS>\n";
		if(modellife>0)
		{
			xml<<"\t<MODEL life=\""<<modellife<<"\" />\n";
		}
		xml<<"</INIT>\n";
		fileout.close();
		file.close();
		return true;
	}
	return false;
}

//--------------------------------------------------------------------------------------


//----------------------------------------------------------
//,    
void GUI::updateBuffer()
{
//	qDebug("updateBuffer %d",isUpdatesEnabled());
	QRect r;
	pixBuffer.fill(backgroundColor);	//    
	QPainter painter(&pixBuffer);
	int x1,x2,y1,y2,xc1,yc1,xc2,yc2;
	double alpha,beta;
	int l=20;
	double PI=3.141592654;
	GuiObjectConnectionsList* p=guiObjectConnectionsList;
	while(p)
	{
		x1=p->obj1->x()+p->obj1->width()/2;
		y1=p->obj1->y()+p->obj1->height()/2;
		x2=p->obj2->x()+p->obj2->width()/2;
		y2=p->obj2->y()+p->obj2->height()/2;
		painter.drawLine(x1,y1,x2,y2);
		xc1=(x1+x2)/2;
		yc1=(y1+y2)/2;
		alpha=atan(static_cast<double>(y1-y2)/static_cast<double>(x2-x1));
		if(x1>x2) alpha=-alpha;
		//qDebug("alpha=%lf",alpha);
		beta=alpha+PI/9.;
		yc2=yc1+static_cast<int>(l*sin(beta));
		
		if(x2>x1) 
			xc2=xc1-static_cast<int>(l*cos(beta));
		else
			xc2=xc1+static_cast<int>(l*cos(beta));
		painter.drawLine(xc1,yc1,xc2,yc2);
		
		beta=alpha-PI/9.;
		yc2=yc1+static_cast<int>(l*sin(beta));
		
		if(x2>x1) 
			xc2=xc1-static_cast<int>(l*cos(beta));
		else
			xc2=xc1+static_cast<int>(l*cos(beta));
		painter.drawLine(xc1,yc1,xc2,yc2);
		p=p->next;
	}
	
//	visualTab->repaint(0,0,-1,-1,FALSE);
	
	visualTab->update();
//	visualTabScroll->resizeContents(visualTab->size().width(),visualTab->size().height());
//	visualTabScroll->updateContents();
}

//----------------------------------------------------------

bool GUI::init(const char* filename)
{
	qDebug("init");
	slotClose();
	
	module=new StructModule(teOutput);
	connect(module,SIGNAL(signalComplete()),this,SLOT(slotComplete()));
	loadCode(filename);
	if(!filename) return false;
	qDebug("-------------------------");
	qDebug("parse \"%s\"",filename);
	QFile file(filename);
	sieInitParser parser(module,this,teOutput);
	QXmlSimpleReader reader;
	reader.setContentHandler(&parser);
	reader.setErrorHandler(&parser);
	if(!reader.parse(&file)) 
	{
		delete module;
		module=0;
		return false;
	}
	qDebug("parsing complete\n-------------------------");
//	module->init(cbVisual->isChecked());
		
	return true;
}

//----------------------------------------------------------

void GUI::resizeEvent(QResizeEvent* revent)
{
	updateBuffer();
}

//----------------------------------------------------------

void GUI::slotPrint()
{
	QPrinter printer;
	printer.setFullPage(TRUE);
	if(printer.setup(this))
	{
		QTextEdit* e=codeTab;
		
		statusBar()->message("...");
		QPainter p;
		if(!p.begin(&printer)) 
		{
			statusBar()->message(" !",2000);
			return;
		}

		QPaintDeviceMetrics metrics(p.device());
		int dpiy=metrics.logicalDpiY();
		int margin=(int)((2/2.54)*dpiy); // 2 cm margins
		QRect body(margin,margin,metrics.width()-2*margin,metrics.height()-2*margin);
		QSimpleRichText richText(QStyleSheet::convertFromPlainText(e->text()),
					  QFont(),
					  e->context(),
					  e->styleSheet(),
					  e->mimeSourceFactory(),
					  body.height() );
		richText.setWidth(&p,body.width());
		QRect view(body);
		int page=1;
		do 
		{
			richText.draw(&p,body.left(),body.top(),view,colorGroup());
			view.moveBy(0,body.height());
			p.translate(0 ,-body.height());
			p.drawText(view.right()-p.fontMetrics().width(QString::number(page)),
				    view.bottom()+p.fontMetrics().ascent()+5,QString::number(page));
			if (view.top()>=richText.height())
				break;
			printer.newPage();
			page++;
		} while(TRUE);

		statusBar()->message(" ",2000);
	} 
	else 
	{
		statusBar()->message(" ",2000);
	}
}

//----------------------------------------------------------

void GUI::slotAbout()
{
	QMessageBox::about(this," ...",
			"  SIE <BR>"
			"<H3 align=center>&quot;StructModule&quot;</H3>"
			": 1.0<BR>"
			":  ..<BR>"
			"<P align=center> . .. <BR><BR>"
			", 2008.</P>");
}

//----------------------------------------------------------

void GUI::slotAboutQt()
{
	QMessageBox::aboutQt(this);
}

