#include <qpainter.h>
#include <qtimer.h>
//--------------------------------------------------------------------------------------
#include "guiobject.h"
#include "gui.h"
//--------------------------------------------------------------------------------------
//#include "ugoproperty.h"
//--------------------------------------------------------------------------------------

GuiObject::GuiObject(QWidget* parent,GuiProperty* prop,const char* name) : 
					QWidget((QWidget*)parent,name),
				active(false),initBackgroundColor(prop->backgroundColor),
				curName(name)
{
//	qDebug("GuiObject(%s)",name);
	
	Property=new GuiProperty;
	*Property=*prop;
//	qDebug("%dx%d",Property->width,Property->height);
	move(Property->x,Property->y);
	
	QFont font(QFont("Times",Property->fontSize));
	QFontMetrics fm(font);
	int w=fm.width(curName)*3/2;
	int h=fm.ascent()*3;

	if(Property->width<w) Property->width=w;
	if(Property->height<h) Property->height=h;
	resize(Property->width,Property->height);
	pixBuffer.resize(size());
	
/*	blinkTimer=new QTimer(this);
	stopBlinkTimer=new QTimer(this);
	connect(blinkTimer,SIGNAL(timeout()),this,SLOT(blinkTic()));
	connect(stopBlinkTimer,SIGNAL(timeout()),this,SLOT(stopBlink()));
*/	
	updateBuffer();
	
} 
 
//--------------------------------------------------------------------------------------

GuiObject::~GuiObject()
{
	qDebug("~GuiObject(%s)",name());
	delete Property;
}

void GuiObject::setInitName(const QString& newname)
{
	curName=newname;
	QFont font(QFont("Times",Property->fontSize));
	QFontMetrics fm(font);
	int w=fm.width(curName)*3/2;
	int h=fm.ascent()*3;

	if(Property->width<w) Property->width=w;
	if(Property->height<h) Property->height=h;
	resize(Property->width,Property->height);
	pixBuffer.resize(size());

	setName(newname);
}
//--------------------------------------------------------------------------------------
void GuiObject::setVisualTransact(int k,bool chName)
{
	if(chName) 
	{
		if(k<=0)
			curName=QString(name());
		else
			curName=QString("%1 (%2)").arg(name()).arg(k);
	}
	
	if(k<=0)
		changeBackgroundColor(initBackgroundColor);
	else
		changeBackgroundColor(initBackgroundColor.dark(100+k*50));
//	qDebug("%s: transact !!!!!!",name());
}
//--------------------------------------------------------------------------------------
//,    
void GuiObject::updateBuffer()
{
	QRect r;
	if(active)
		pixBuffer.fill(Property->backgroundColor.light());	//    
	else
		pixBuffer.fill(Property->backgroundColor);	//    
	QPainter painter(&pixBuffer);
	
	painter.setPen(QPen(black,1));
	r.setRect(0,0,Property->width,Property->height);
	painter.drawRect(r);			//   
	
	painter.setPen(QPen(Property->textColor));
	QFont font(QFont("Times",Property->fontSize));
	painter.setFont(font);
	QFontMetrics fm(font);
	int x=(Property->width-fm.width(curName))/2;	// 
	int y=(Property->height+fm.ascent()/2)/2;	// 
	painter.drawText(x,y,curName);		// 
	repaint();	
}

//--------------------------------------------------------------------------------------
/*
void UGO::changeFrameColor(const QColor& color)
{
	frameColor=color;
	updateBuffer();
}
*/
//--------------------------------------------------------------------------------------

void GuiObject::changeBackgroundColor(const QColor& color)
{
	Property->backgroundColor=color;
	updateBuffer();
}

//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------

void GuiObject::paintEvent(QPaintEvent* pevent)
{
	bitBlt(this,0,0,&pixBuffer);
}

//--------------------------------------------------------------------------------------

void GuiObject::mousePressEvent(QMouseEvent* pe)
{
	if(pe->button()==LeftButton)
	{
		allowToMove=true;
		dX=pe->globalX()-x();
		dY=pe->globalY()-y();
//		active=true;
		emit signalPress(this);
//		qDebug("press event");
//		updateBuffer();
	}
	else if(pe->button()==RightButton)
	{
	/*	startX=pe->x();
		startY=pe->y();
		startPoint=pe->globalPos();
		startPoint.setX(startPoint.x()-pe->x());
		emit signalPress(startPoint);
	*/
	}
}

void GuiObject::slotSetInactive()
{
	if(!active) return;
//	qDebug("%s: set inactive",name());
	active=false;
	updateBuffer();
}

void GuiObject::slotSetActive()
{
	if(active) return;
	active=true;
	updateBuffer();
}

//--------------------------------------------------------------------------------------

void GuiObject::mouseReleaseEvent(QMouseEvent* pe)
{
	if(pe->button()==LeftButton)
	{
		allowToMove=false;
	}
	else if(pe->button()==RightButton)
	{
		/*
		if(startPoint.x()*startPoint.y()>0)
		{
			QPainter painter(this);
			painter.setPen(QPen(red,10));
//			painter.drawLine(mapFromGlobal(startPoint),pe->pos());
			emit signalRelease(startPoint,pe->globalPos());
			
		}
		startPoint=QPoint(0,0);
	*/
	}
}

//--------------------------------------------------------------------------------------

void GuiObject::mouseMoveEvent(QMouseEvent* pe)
{
	if(allowToMove)
	{
		
		int xx=pe->globalX()-dX;
		int yy=pe->globalY()-dY;
		
			
		int w=parentWidget()->width()-width();
		int h=parentWidget()->height()-height();
		if(xx<0) xx=0;
		if(yy<0) yy=0;
		if(xx>w) xx=w;
		if(yy>h) yy=h;
		move(xx,yy);

		emit signalFix(this);
	}
}

//--------------------------------------------------------------------------------------

void GuiObject::mouseDoubleClickEvent(QMouseEvent* pe)
{
	if(pe->button()==LeftButton)
	{
		emit signalDown(this);
	}
	else if(pe->button()==RightButton)
	{
		emit signalKill(this);
	}
}

//--------------------------------------------------------------------------------------
