#include <qapplication.h>
#include <qtextedit.h>

#ifdef WIN32
#include <qtextcodec.h>
#endif

#include "sieinitparser.h"
#include "structmodule.h"
#include "siegenerator.h"
#include "gui.h"
#include "sietypes.h"
//--------------------------------------------------------------------------

sieInitParser::sieInitParser(StructModule* mod,GUI* canv,QTextEdit* out) : 
				module(mod),canvas(canv),conNum(0),
				errStr(""),output(out)
{
	memset(&init,0,sizeof(init));	
//	qDebug("sizeof(init)=%d",sizeof(init));
}

//--------------------------------------------------------------------------

sieInitParser::~sieInitParser()
{
}

//--------------------------------------------------------------------------

bool sieInitParser::startElement(const QString &,
			const QString &,
			const QString &qName,
			const QXmlAttributes &attrib)
{
//	QTextCodec *codec = QTextCodec::codecForName("KOI8-R");
//	QString qName = codec->fromUnicode(qName1251);
	
	
	if(qName=="INIT")
	{
		return true;
	}
	else if(qName=="OBJECTS")
	{
//		init.objects=true;
		return true;
	}
	else if(qName=="GENERATOR")
	{
		return parseObject(SIE_GENERATOR,attrib);
	}
	else if(qName=="DEVICE")
	{
		return parseObject(SIE_DEVICE,attrib);
	}
	else if(qName=="TERMINATOR")
	{
		return parseObject(SIE_TERMINATOR,attrib);
	}
	else if(qName=="QUEUE")
	{
		return parseObject(SIE_QUEUE,attrib);
	}
	else if(qName=="PACKER")
	{
		return parseObject(SIE_PACKER,attrib);
	}
	else if(qName=="UNPACKER")
	{
		return parseObject(SIE_UNPACKER,attrib);
	}
	else if(qName=="SELECTOR")
	{
		return parseObject(SIE_SELECTOR,attrib);
	}
	else if(qName=="GUI")
	{
		return parseGui(attrib);
	}
	else if(qName=="TRANSACT")
	{
		return parseTransact(attrib);
	}
	else if(qName=="CONNECTION")
	{
		return parseConnection(attrib);
	}
	else if(qName=="MODEL")
	{
		return parseModel(attrib);
	}
	
	errStr=QString("   \"%1\"").arg(qName);
	return false;
}

//--------------------------------------------------------------------------

bool sieInitParser::endElement(const QString &,
			const QString &,
			const QString &qName)
{
	if(qName=="INIT")
	{
		return true;
	}
	else if(qName=="OBJECTS")
	{
//		init.objects=false;
		QString str;
		for(int i=0;i<conNum;i++)
		{
			bool ok=module->setConnection(conList[i].obj1,conList[i].obj2,
					conList[i].type,conList[i].probability);
			printf("connect(%s,%s)  type=%d  \tok=%d\n",conList[i].obj1,conList[i].obj2,conList[i].type,ok);
			str=QString("connect(%1,%2)  type=%3").arg(conList[i].obj1).arg(conList[i].obj2).arg(conList[i].type);
			if(ok) str+="\tok";
			//output->append(str);
			canvas->addGuiObjectConnection(conList[i].obj1,conList[i].obj2);
		}
		canvas->updateBuffer();
		return true;
	}
	else if(qName=="TRANSACT")
	{
		return true;
	}
	else if(qName=="GENERATOR")
	{
		dynamic_cast<sieGenerator*>(init.currentObject.obj)->setTransactProperty(&init.transact,
			init.trLLD,init.trCD);
		return true;
	}
	else if(qName=="QUEUE")
	{
		return true;
	}
	else if(qName=="PACKER")
	{
		return true;
	}
	else if(qName=="UNPACKER")
	{
		return true;
	}
	else if(qName=="GUI")
	{
		//init.generator=false;
		return true;
	}
	else if(qName=="CONNECTION")
	{
		//init.generator=false;
		return true;
	}
	else if(qName=="MODEL")
	{
		//init.generator=false;
		return true;
	}
	else if(qName=="TERMINATOR")
	{
//		init.terminator=false;
		return true;
	}
	else if(qName=="DEVICE")
	{
		return true;
	}
	else if(qName=="SELECTOR")
	{
		return true;
	}
	errStr=QString("   \"/%1\"").arg(qName);
	return false;
}

//--------------------------------------------------------------------------

//--------------------------------------------------------------------------

bool sieInitParser::parseObject(int objectType,const QXmlAttributes &attrib)
{
	int i;
	bool err=false;
	int len=attrib.length();
	char name[50]="";
	int type=-1;
	int tgen=-1;
	int ngen=-1;
	int maxnum=-1;
	int delay=-1;
	int disp=-1;
	int numdec=-1;
#ifdef WIN32
	QTextCodec *codec = QTextCodec::codecForName("CP1251");
	QString strname;
#endif
	//strcpy(conList[conNum].obj1,name);
	for(i=0;i<len;i++)
	{
		if(attrib.qName(i)=="name")
		{
#ifdef WIN32
			strname = codec->toUnicode(attrib.value(i));
			strcpy(name,strname.ascii());
#else
			strcpy(name,attrib.value(i).ascii());
#endif
			strcpy(init.currentObject.name,name);
			printf("load %s\n",name);
		}
		else if(attrib.qName(i)=="type")
			type=attrib.value(i).toInt();
		else if((objectType==SIE_GENERATOR)&&(attrib.qName(i)=="tgen"))
			tgen=attrib.value(i).toInt();
		else if((objectType==SIE_GENERATOR)&&(attrib.qName(i)=="disp"))
			disp=attrib.value(i).toInt();
		else if((objectType==SIE_GENERATOR)&&(attrib.qName(i)=="ngen"))
			ngen=attrib.value(i).toInt();
		else if((objectType==SIE_QUEUE)&&(attrib.qName(i)=="maxnum"))
			maxnum=attrib.value(i).toInt();
		else if((objectType==SIE_DEVICE)&&(attrib.qName(i)=="delay"))
			delay=attrib.value(i).toInt();
		else if((objectType==SIE_DEVICE)&&(attrib.qName(i)=="disp"))
			disp=attrib.value(i).toInt();
		else if((objectType==SIE_TERMINATOR)&&(attrib.qName(i)=="decrement"))
			numdec=attrib.value(i).toInt();
//		else if(attrib.qName(i)=="background")
//			background=QColor(attrib.value(i).ascii());
		else
		{
			err=true;
			break;
		}
	}
	
	if(err)
	{
		errStr=QString("   (%1) %2").arg(attrib.qName(i)).arg(objectType);
		return false;
	}
	
	if(name[0]=='\0')
	{
		errStr=QString("  ");
		return false;
	}

	
	if((objectType==SIE_GENERATOR)&&(tgen>0))
	{
		///new generator
		init.currentObject.type=SIE_GENERATOR;
		init.currentObject.obj=module->addObject(name,SIE_GENERATOR,tgen,disp,ngen);
	}
	else if(objectType==SIE_TERMINATOR)
	{
		///new terminator
		init.currentObject.type=SIE_TERMINATOR;
		init.currentObject.obj=module->addObject(name,SIE_TERMINATOR,numdec);

	}
	else if((objectType==SIE_QUEUE)&&(maxnum>0))
	{
		///new queue
		init.currentObject.type=SIE_QUEUE;
		init.currentObject.obj=module->addObject(name,SIE_QUEUE,maxnum);

	}
	else if((objectType==SIE_DEVICE)&&(delay>0))
	{
		///new device
		init.currentObject.type=SIE_DEVICE;
		init.currentObject.obj=module->addObject(name,SIE_DEVICE,delay,disp);

	}
	else if((objectType==SIE_PACKER)&&(type>=0))
	{
		///new packer
		init.currentObject.type=SIE_PACKER;
		init.currentObject.obj=module->addObject(name,SIE_PACKER,type);

	}
	else if(objectType==SIE_UNPACKER)
	{
		///new unpacker
		init.currentObject.type=SIE_UNPACKER;
		init.currentObject.obj=module->addObject(name,SIE_UNPACKER,type);
	}
	else if(objectType==SIE_SELECTOR)
	{
		///new selector
		init.currentObject.type=SIE_SELECTOR;
		init.currentObject.obj=module->addObject(name,SIE_SELECTOR,type);
	}
	else 
	{
		return false;
	}
	
	
	return true;
}

//--------------------------------------------------------------------------
bool sieInitParser::parseConnection(const QXmlAttributes &attrib)
{
	int i;
	bool err=false;
	int len=attrib.length();
	conList[conNum].type=-1;
	conList[conNum].probability=-1;
	#ifdef WIN32
		QTextCodec *codec = QTextCodec::codecForName("CP1251");
		QString strname;
	#endif

	for(i=0;i<len;i++)
	{
		if(attrib.qName(i)=="destination")
		{
			strcpy(conList[conNum].obj1,init.currentObject.name);


#ifdef WIN32
			strname = codec->toUnicode(attrib.value(i));
			strcpy(conList[conNum].obj2,strname.ascii());
#else
			strcpy(conList[conNum].obj2,attrib.value(i).ascii());
#endif

		}
		else if(attrib.qName(i)=="type")
			conList[conNum].type=attrib.value(i).toInt();
		else if(attrib.qName(i)=="probability")
			conList[conNum].probability=attrib.value(i).toInt();
		else
		{
			err=true;
			break;
		}
	}
	
	if(err)
	{
		errStr=QString("   (%1)").arg(attrib.qName(i));
		return false;
	}
	
	conNum++;
	return true;
}

bool sieInitParser::parseTransact(const QXmlAttributes &attrib)
{
	int i;
	bool err=false;
	int len=attrib.length();
	
	for(i=0;i<len;i++)
	{
		if(attrib.qName(i)=="type")
			init.transact.type=attrib.value(i).toInt();
		else if(attrib.qName(i)=="lifelimit")
			init.transact.lifeLimit=attrib.value(i).toInt();
		else if(attrib.qName(i)=="lifelimitdisp")
			init.trLLD=attrib.value(i).toInt();
		else if(attrib.qName(i)=="cost")
			init.transact.cost=attrib.value(i).toDouble();
		else if(attrib.qName(i)=="costdisp")
			init.trCD=attrib.value(i).toDouble();
		else
		{
			err=true;
			break;
		}
	}
	
	if(err)
	{
		errStr=QString("   (%1)").arg(attrib.qName(i));
		return false;
	}
	
	return true;
}
//--------------------------------------------------------------------------
bool sieInitParser::parseGui(const QXmlAttributes &attrib)
{
	int x=100;
	int y=100;
	int i;
	bool err=false;
	
	int len=attrib.length();
	
	for(i=0;i<len;i++)
	{
		if(attrib.qName(i)=="X")
			x=attrib.value(i).toInt();
		else if(attrib.qName(i)=="Y")
			y=attrib.value(i).toInt();
		else
		{
			err=true;
			break;
		}
	}
	
	if((x<0)||(y<0)||err)
	{
		errStr=QString("   (%1)").arg(attrib.qName(i));
		return false;
	}
	
	init.currentObject.x=x;
	init.currentObject.y=y;
	
	
	GuiObject* gobj=canvas->addGuiObject(init.currentObject.type,init.currentObject.name,
			     init.currentObject.x,init.currentObject.y);
	module->setCoordinates(init.currentObject.name,x,y);
	
//	output->append(QString("Gui object = %1").arg((long int)gobj)); 
	init.currentObject.obj->setGui(gobj);

	return true;
}

//--------------------------------------------------------------------------
bool sieInitParser::parseModel(const QXmlAttributes &attrib)
{
	int i;
	bool err=false;
	int len=attrib.length();
	
	for(i=0;i<len;i++)
	{
		if(attrib.qName(i)=="life")
		{
			module->setLife(attrib.value(i).toInt());
		}
		else
		{
			err=true;
			break;
		}
	}
	
	if(err)
	{
		errStr=QString("   (%1)").arg(attrib.qName(i));
		return false;
	}
	return true;
}


//--------------------------------------------------------------------------

bool sieInitParser::characters(const QString &)
{
	return true;
}

//--------------------------------------------------------------------------
bool sieInitParser::fatalError(const QXmlParseException &exc)
{
	printf("%d:%d fatal error: %s\n",exc.lineNumber(),
			exc.columnNumber(),exc.message().ascii());
	output->append(QString("%1:%2 fatal error: %3").arg(exc.lineNumber()).arg(exc.columnNumber()).arg(exc.message().ascii()));
	return true;
}

//--------------------------------------------------------------------------

QString sieInitParser::errorString()
{
	return errStr;
}

//--------------------------------------------------------------------------


